#!/usr/bin/python

################################################################
#                                                              #
# Blue Team Training Toolkit (BT3)                             #
# written by Juan J. Guelfo @ Encripto AS                      #
# post@encripto.no                                             #
#                                                              #
# Copyright 2013-2018 Encripto AS. All rights reserved.        #
#                                                              #
# BT3 is licensed under the FreeBSD license.                   #
# http://www.freebsd.org/copyright/freebsd-license.html        #
#                                                              #
################################################################


import cmd, os
import importlib, zipfile
import libs.bt3out, libs.bt3in, libs.bt3api, libs.bt3com


__module_name__    = "Mocksum"
__module_version__ = "1.3"


class Options():

    def __init__(self):
        self.options = ""

    def get_options(self):
        self.options = [["DUMMY", str(" "), "False", "Dummy option"]]

        return self.options


    def validate_options(self):
        print("")
        return self.validate_mandatory_args() and self.validate_optional_args()


    def validate_mandatory_args(self):
        valid_options = True
        return valid_options


    def validate_optional_args(self):
        valid_options = True
        return valid_options



class Menu(cmd.Cmd):

    def __init__(self):
        cmd.Cmd.__init__(self)
        self.api_credentials = []
        self.resource_cmds = []
        self.config = Options()
        self.mocksum = Mocksum()
        self.bytes_measure = 1000000.0
        self.description_length = 100
        self.mock_folder = "./mockfiles"
        libs.bt3out.setup_material_folder(self.mock_folder)


    def clear_api_creds(self):
        if len(self.api_credentials) == 2:
            self.api_credentials[0] = libs.bt3api.generate_string()
            self.api_credentials[1] = libs.bt3api.generate_string()
            self.api_credentials    = []


    def complete_download(self, text, line, begidx, endidx):
        return libs.bt3com.autocomplete_download(self.api_credentials, text, line, "mockfile")


    def complete_info(self, text, line, begidx, endidx):
        return libs.bt3com.autocomplete_info(self.api_credentials, text, line, "mockfile", self.mock_folder)


    def complete_show(self, text, line, begidx, endidx):
        return libs.bt3com.autocomplete_show(text, line, "mockfile")


    def default(self, args):
        print("")
        libs.bt3out.print_error("%s\n" % libs.bt3out.INVALID_COMMAND)


    def do_download(self, args):
        libs.bt3com.cmd_download(args, self.api_credentials, "mockfile", self.mock_folder)


    def emptyline(self):
        pass


    def do_back(self, args):
        return True


    def do_exit(self, args):
        self.clear_api_creds()
        libs.bt3out.exit_program(args)


    def do_help(self, args):
        cmds = [["back", "Exit current selected module and return to main menu."],
                ["download <mockfile>", "Download a given mock file from the Blue Team Training Toolkit cloud."],
                ["exit", "Exit the Blue Team Training Toolkit."],
                ["help", "Display help menu."],
                ["info <mockfile>", "Display detailed information about a mock file."],
                ["run", "Run the module with the given options."],
                ["search <string>", "Find mock files based on a given string."],
                ["set <option> <value>", "Set module option."],
                ["show downloads", "Display a history of mock files downloaded from the cloud."],
                ["show interfaces", "Display available network interfaces."],
                ["show mockfiles", "Display all available mock files."],
                ["show mockfiles cloud", "Display mock files available in the cloud."],
                ["show mockfiles disk", "Display mock files available on your computer."],
                ["show mockfiles free", "Display free mock files available in the cloud."],
                ["show mockfiles premium", "Display premium mock files available in the cloud."],
                ["show options", "Display module options."],
                ["version", "Display module version."]]
        libs.bt3out.print_help(cmds, False, True)


    def do_info(self, args):
        args = args.split(" ")
        if len(args) == 1 and args[0] not in [""]:
            try:
                mock_folder = self.mock_folder
                result = []
                mockfile = args[0]
                if mockfile.endswith(".mock") and libs.bt3in.check_file_exists("%s/%s" % (mock_folder, os.path.basename(mockfile))):
                    mockfile_name = mockfile
                    mockfile_size = os.path.getsize("%s/%s" % (mock_folder, os.path.basename(mockfile_name)))
                    mockfile_location = "Disk"
                    mockfile_date = libs.bt3out.NO_DATA
                    mockfile_author = libs.bt3out.NO_DATA
                    mockfile_description = libs.bt3out.NO_DATA
                    mockfile_references = libs.bt3out.NO_DATA

                    mock_module_file = "%s.py" % mockfile.replace(".mock", "")
                    if mock_module_file not in ["__init__.py"] and libs.bt3in.check_file_exists("%s/%s" % (mock_folder, os.path.basename(mock_module_file))):
                        loaded_mockfile = self.mocksum.load_info(mock_module_file.replace(".py", ""))

                        if len(loaded_mockfile.info.date) > 0:
                            mockfile_date = loaded_mockfile.info.date

                        if len(loaded_mockfile.info.author) > 0:
                            mockfile_author = loaded_mockfile.info.author

                        if len(loaded_mockfile.info.references) > 0:
                            mockfile_references = loaded_mockfile.info.references

                        if len(loaded_mockfile.info.description) > 0:
                            mockfile_description = loaded_mockfile.info.description

                    result.append([ mockfile_name, str(round(mockfile_size / self.bytes_measure, 3)), mockfile_location, mockfile_date, " ", mockfile_description.replace("\n", "")[:self.description_length], mockfile_references, mockfile_author ])

                #API INTEGRATION - START
                if len(self.api_credentials) == 2 and len(result) == 0:
                    json_results = libs.bt3api.get_material(self.api_credentials[0], self.api_credentials[1], "mockfile", args[0])
                    if json_results and libs.bt3api.validate_json(json_results):
                        parsed_results = libs.bt3api.parse_json(json_results)
                        if parsed_results[-1]["Result"]:
                            del parsed_results[-1]
                            for mockfile in parsed_results:
                                result.append([ mockfile["Name"], mockfile["Size"], "Cloud", mockfile["Date"], mockfile["Price"], mockfile["Description"].replace("\n", ""), mockfile["References"], mockfile["Author"] ])
                    else:
                        print("")
                        libs.bt3out.print_error("%s" % libs.bt3out.API_INCOMMUNICATION)
                        libs.bt3out.print_error("%s\n" % libs.bt3out.CONTACT_SUPPORT)

                #API INTEGRATION - END

                if len(result) > 0:
                    libs.bt3out.print_material_details(result, "Mock File")

                else:
                    print("")
                    libs.bt3out.print_info("%s\n" % libs.bt3out.NO_MOCKFILES_FOUND)

            except Exception as e:
                print("")
                libs.bt3out.print_error("%s\n" % e)

        else:
            cmds = [["info <mockfile>", "Display detailed information about a mock file."]]
            libs.bt3out.print_help(cmds, True, True)


    def preloop(self):
        if len(self.resource_cmds) > 0:
            for cmd in self.resource_cmds:
                c = self.precmd(cmd)
                e = self.onecmd(c)

            self.resource_cmds = []


    def do_run(self, args):
        print("")
        libs.bt3out.print_error("%s\n" % libs.bt3out.NO_RUN_COMMAND)


    def do_search(self, args):
        if args not in [""]:
            try:
                mock_folder = self.mock_folder
                files = [f for f in os.listdir(mock_folder) if os.path.isfile(os.path.join(mock_folder, f))]
                files.sort()
                result = []
                for mockfile in files:
                    if mockfile.endswith(".mock") and libs.bt3in.check_file_exists("%s/%s" % (mock_folder, os.path.basename(mockfile))):
                        mockfile_name = mockfile
                        mockfile_size = os.path.getsize("%s/%s" % (mock_folder, os.path.basename(mockfile_name)))
                        mockfile_location = "Disk"
                        mockfile_date = libs.bt3out.NO_DATA
                        mockfile_description = libs.bt3out.NO_DATA

                        mockfile_module_file = "%s.py" % mockfile.replace(".mock", "")
                        if mockfile_module_file not in ["__init__.py"] and libs.bt3in.check_file_exists("%s/%s" % (mock_folder, os.path.basename(mockfile_module_file))):
                            loaded_mockfile = self.mocksum.load_info(mockfile_module_file.replace(".py", ""))

                            if len(loaded_mockfile.info.date) > 0:
                                mockfile_date = loaded_mockfile.info.date

                            if len(loaded_mockfile.info.description) > 0:
                                mockfile_description = loaded_mockfile.info.description

                        if mockfile_name.upper().find(args.upper()) > -1 or mockfile_description.upper().find(args.upper()) > -1:
                            result.append([ mockfile_name, str(round(mockfile_size / self.bytes_measure, 3)), mockfile_location, mockfile_date, " ", mockfile_description.replace("\n", "")[:self.description_length] ])

                #API INTEGRATION - START
                if len(self.api_credentials) == 2:
                    json_results = libs.bt3api.get_material_list(self.api_credentials[0], self.api_credentials[1], "mockfile")
                    if json_results and libs.bt3api.validate_json(json_results):
                        parsed_results = libs.bt3api.parse_json(json_results)
                        if parsed_results[-1]["Result"]:
                            del parsed_results[-1]
                            for mockfile in parsed_results:
                                if mockfile["Name"].upper().find(args.upper()) > -1 or mockfile["Description"].upper().find(args.upper()) > -1:
                                    result.append([ mockfile["Name"],  mockfile["Size"], "Cloud", mockfile["Date"], mockfile["Price"], mockfile["Description"].replace("\n", "") ])

                    else:
                        print("")
                        libs.bt3out.print_error("%s" % libs.bt3out.API_INCOMMUNICATION)
                        libs.bt3out.print_error("%s\n" % libs.bt3out.CONTACT_SUPPORT)

                #API INTEGRATION - END

                if len(result) > 0:
                    libs.bt3out.print_list(result)
                    print("")
                    libs.bt3out.print_info("Search results: %s\n" % len(result))

                else:
                    print("")
                    libs.bt3out.print_info("%s\n" % libs.bt3out.NO_MOCKFILES_FOUND)

            except Exception as e:
                print("")
                libs.bt3out.print_error("%s\n" % e)

        else:
            cmds = [["search <string>", "Find mock files based on a given string."]]
            libs.bt3out.print_help(cmds, True, True)


    def do_set(self, args):
        print("")
        libs.bt3out.print_info("%s\n" % libs.bt3out.NO_OPTIONS)


    def do_show(self, args):
        if args.upper() in ["DOWNLOADS"]:
            libs.bt3com.cmd_show_downloads(self.api_credentials, "mockfile")

        elif args.upper() in ["INTERFACES"]:
            libs.bt3out.print_interfaces()

        elif args.upper() in ["OPTIONS"]:
            print("")
            libs.bt3out.print_info("%s\n" % libs.bt3out.NO_OPTIONS)

        elif args.upper() in ["MOCKFILES", "MOCKFILES CLOUD", "MOCKFILES DISK", "MOCKFILES FREE", "MOCKFILES PREMIUM"]:
            try:
                mock_folder = self.mock_folder
                files = [f for f in os.listdir(mock_folder) if os.path.isfile(os.path.join(mock_folder, f))]
                files.sort()
                result = []

                if args.upper() in ["MOCKFILES", "MOCKFILES DISK"]:
                    for mockfile in files:
                        if mockfile.endswith(".mock") and libs.bt3in.check_file_exists("%s/%s" % (mock_folder, os.path.basename(mockfile))):
                            mockfile_name = mockfile
                            mockfile_size = os.path.getsize("%s/%s" % (mock_folder, os.path.basename(mockfile_name)))
                            mockfile_location = "Disk"
                            mockfile_date = libs.bt3out.NO_DATA
                            mockfile_description = libs.bt3out.NO_DATA

                            mockfile_module_file = "%s.py" % mockfile.replace(".mock", "")
                            if mockfile_module_file not in ["__init__.py"] and libs.bt3in.check_file_exists("%s/%s" % (mock_folder, os.path.basename(mockfile_module_file))):
                                loaded_mockfile = self.mocksum.load_info(mockfile_module_file.replace(".py", ""))

                                if len(loaded_mockfile.info.date) > 0:
                                    mockfile_date = loaded_mockfile.info.date

                                if len(loaded_mockfile.info.description) > 0:
                                    mockfile_description = loaded_mockfile.info.description

                            result.append([ mockfile_name, str(round(mockfile_size / self.bytes_measure, 3)), mockfile_location, mockfile_date, " ", mockfile_description.replace("\n", "")[:self.description_length] ])

                #API INTEGRATION - START
                api_error = False
                if args.upper() in ["MOCKFILES", "MOCKFILES CLOUD", "MOCKFILES FREE", "MOCKFILES PREMIUM"]:
                    if len(self.api_credentials) == 2:
                        json_results = libs.bt3api.get_material_list(self.api_credentials[0], self.api_credentials[1], "mockfile")
                        if json_results and libs.bt3api.validate_json(json_results):
                            parsed_results = libs.bt3api.parse_json(json_results)
                            if parsed_results[-1]["Result"]:
                                del parsed_results[-1]
                                for mockfile in parsed_results:
                                    if args.upper() in ["MOCKFILES", "MOCKFILES CLOUD"]:
                                        result.append([ mockfile["Name"], mockfile["Size"], "Cloud", mockfile["Date"], mockfile["Price"], mockfile["Description"].replace("\n", "") ])

                                    elif args.upper() in ["MOCKFILES FREE"] and mockfile["Price"] == "0":
                                        result.append([ mockfile["Name"], mockfile["Size"], "Cloud", mockfile["Date"], mockfile["Price"], mockfile["Description"].replace("\n", "") ])

                                    elif args.upper() in ["MOCKFILES PREMIUM"] and mockfile["Price"] != "0":
                                        result.append([ mockfile["Name"], mockfile["Size"], "Cloud", mockfile["Date"], mockfile["Price"], mockfile["Description"].replace("\n", "") ])

                        else:
                            print("")
                            api_error = True
                            libs.bt3out.print_error("%s" % libs.bt3out.API_INCOMMUNICATION)
                            libs.bt3out.print_error("%s\n" % libs.bt3out.CONTACT_SUPPORT)

                    else:
                        if args.upper() in ["MOCKFILES CLOUD", "MOCKFILES FREE", "MOCKFILES PREMIUM"]:
                            print("")
                            api_error = True
                            libs.bt3out.print_error("%s\n" % libs.bt3out.API_AUTHENTICATION)

                #API INTEGRATION - END

                if len(result) > 0:
                    libs.bt3out.print_list(result)
                    print("")
                    libs.bt3out.print_info("Available mock files: %s\n" % len(result))

                elif not api_error:
                    print("")
                    libs.bt3out.print_info("%s\n" % libs.bt3out.NO_MOCKFILES_FOUND)

            except Exception as e:
                print("")
                libs.bt3out.print_error("%s\n" % e)


        else:
            cmds = [["show downloads", "Display a history of mock files downloaded from the cloud."],
                    ["show interfaces", "Display available network interfaces."],
                    ["show mockfiles", "Display all available mock files."],
                    ["show mockfiles cloud", "Display mock files available in the cloud."],
                    ["show mockfiles disk", "Display mock files available on your computer."],
                    ["show mockfiles free", "Display free mock files available in the cloud."],
                    ["show mockfiles premium", "Display premium mock files available in the cloud."],
                    ["show options", "Display module options."]]
            libs.bt3out.print_help(cmds, True, True)


    def do_version(self, args):
        print("")
        libs.bt3out.print_info("You are running %s version %s\n" % (__module_name__, __module_version__))



class Mocksum():

    def load_info(self, mockfile):
        mockfile_module = importlib.import_module("mockfiles.%s" % mockfile)
        mockfile_module = reload(mockfile_module)

        return mockfile_module.Mockfile()

